"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _graphqlTag = _interopRequireDefault(require("graphql-tag"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireDefault(require("path"));

var _crypto = _interopRequireDefault(require("crypto"));

var _glob = _interopRequireDefault(require("glob"));

var _util = _interopRequireDefault(require("util"));

var _chalk = _interopRequireDefault(require("chalk"));

var _debug = _interopRequireDefault(require("debug"));

var _lodash = _interopRequireDefault(require("lodash.get"));

var _request = _interopRequireDefault(require("request"));

var _mimeTypes = _interopRequireDefault(require("mime-types"));

var _createApi = _interopRequireDefault(require("./createApi"));

var _SdkError = _interopRequireDefault(require("./SdkError"));

const log = (0, _debug.default)("wcc");

const globFiles = _util.default.promisify(_glob.default);

class Client {
  constructor({
    token = ""
  }) {
    (0, _defineProperty2.default)(this, "api", void 0);
    this.api = (0, _createApi.default)({
      token,
      uri: process.env.WEBINY_CLOUD_API || "https://cloud.webiny.com/api",
      defaultOptions: {
        query: {
          fetchPolicy: "network-only",
          errorPolicy: "all"
        }
      }
    });
  }

  log(...args) {
    log(...args);
  }

  async whoami() {
    const query = _graphqlTag.default`
            {
                currentUser {
                    data {
                        email
                    }
                    error {
                        code
                        data
                        message
                    }
                }
            }
        `;
    let result;

    try {
      result = await this.api.query({
        query
      });
    } catch (err) {
      throw err.networkError ? new _SdkError.default(err.networkError) : err;
    }

    const {
      data,
      error
    } = (0, _lodash.default)(result, "data.currentUser", {});

    if (error) {
      throw new _SdkError.default(error);
    }

    return data;
  }

  async sites() {
    const query = _graphqlTag.default`
            {
                sites(perPage: 100) {
                    data {
                        id
                        name
                        freeHostname
                        customHostname
                    }
                    error {
                        code
                        data
                        message
                    }
                }
            }
        `;
    let result;

    try {
      result = await this.api.query({
        query
      });
    } catch (err) {
      throw err.networkError ? new _SdkError.default(err.networkError) : err;
    }

    const {
      data,
      error
    } = (0, _lodash.default)(result, "data.sites", {});

    if (error) {
      throw new _SdkError.default(error);
    }

    return data;
  }

  async isDeployActive(id) {
    const query = _graphqlTag.default`
            query IsDeployActive($id: ID!) {
                isDeployActive(id: $id) {
                    data {
                        active
                        url
                    }
                    error {
                        code
                        message
                        data
                    }
                }
            }
        `;
    let result;

    try {
      result = await this.api.query({
        query,
        variables: {
          id
        }
      });
    } catch (err) {
      throw err.networkError ? new _SdkError.default(err.networkError) : err;
    }

    const {
      data,
      error
    } = (0, _lodash.default)(result, "data.isDeployActive", {});

    if (error) {
      throw new _SdkError.default(error);
    }

    return data;
  }

  async getFilesDigest(dir) {
    log(`Reading files from %o`, dir);
    const filePaths = await globFiles("**/*", {
      cwd: dir,
      nodir: true
    });
    const files = filePaths.map(file => {
      return {
        abs: _path.default.join(dir, file),
        key: file,
        type: _mimeTypes.default.lookup(_path.default.join(dir, file))
      };
    });
    await Promise.all(files.map(async file => {
      file.hash = await this.getFileHash(file.abs);
    }));
    return files;
  }

  async getFileHash(file) {
    const content = await _fsExtra.default.readFile(file);

    const sha = _crypto.default.createHash("sha1");

    sha.update(content);
    return sha.digest("hex");
  }

  async createDeploy(site, type, path, files, meta = {}) {
    log(`Creating ${_chalk.default.blue(type)} deploy for path ${_chalk.default.blue(path)}`);
    const deployFiles = files.map(({
      key,
      hash,
      type
    }) => {
      log("%s - %s", hash, key);
      return {
        key,
        hash,
        type
      };
    });
    const mutation = _graphqlTag.default`
            mutation CreateDeploy(
                $site: ID!
                $type: DeployType!
                $path: String!
                $files: [DeployFileInput]!
                $meta: DeployMetaInput
            ) {
                createDeploy(site: $site, type: $type, path: $path, files: $files, meta: $meta) {
                    data {
                        id
                        files {
                            key
                            type
                            hash
                            required
                        }
                    }
                    error {
                        code
                        message
                        data
                    }
                }
            }
        `;
    let result;

    try {
      const variables = {
        site,
        type,
        path,
        files: deployFiles,
        meta
      };
      result = await this.api.mutate({
        mutation,
        variables
      });
    } catch (err) {
      throw err.networkError ? new _SdkError.default(err.networkError) : err;
    }

    const {
      data,
      error
    } = (0, _lodash.default)(result, "data.createDeploy", {});

    if (error) {
      throw new _SdkError.default(error);
    }

    return data;
  }

  async activateDeploy(deployId) {
    const mutation = _graphqlTag.default`
            mutation ActivateDeploy($deploy: ID!) {
                activateDeploy(id: $deploy) {
                    data
                    error {
                        code
                        message
                    }
                }
            }
        `;
    let result;

    try {
      const variables = {
        deploy: deployId
      };
      result = await this.api.mutate({
        mutation,
        variables
      });
    } catch (err) {
      throw err.networkError ? new _SdkError.default(err.networkError) : err;
    }

    const {
      data,
      error
    } = (0, _lodash.default)(result, "data.activateDeploy", {});

    if (error) {
      throw new _SdkError.default(error);
    }

    return data;
  }

  async presignFiles(siteId, deployId, files) {
    const query = _graphqlTag.default`
            query PresignFiles($siteId: ID!, $deployId: ID!, $files: [PresignedFileInput]) {
                presignFiles(site: $siteId, deploy: $deployId, files: $files) {
                    data {
                        key
                        presigned
                    }
                    error {
                        code
                        message
                    }
                }
            }
        `;
    let result;

    try {
      result = await this.api.query({
        query,
        variables: {
          siteId,
          deployId,
          files
        }
      });
    } catch (err) {
      throw err.networkError ? new _SdkError.default(err.networkError) : err;
    }

    const {
      data,
      error
    } = (0, _lodash.default)(result, "data.presignFiles", {});

    if (error) {
      throw new _SdkError.default(error);
    }

    return data;
  }

  async uploadPresignedFile({
    url,
    fields
  }, content) {
    const options = {
      method: "POST",
      url,
      formData: (0, _objectSpread2.default)({}, fields, {
        file: content
      })
    };
    return new Promise((resolve, reject) => {
      (0, _request.default)(options, error => {
        if (error) {
          return reject(error);
        }

        resolve();
      });
    });
  }

}

exports.default = Client;
//# sourceMappingURL=client.js.map